//
// Scheduler.h
//
// Copyright (c) Shareaza Development Team, 2002-2007.
// This file is part of SHAREAZA (shareaza.sourceforge.net)
//
// Shareaza is free software; you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// Shareaza is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Shareaza; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#if !defined(AFX_SCHEDULER_H__INCLUDED_)
#define AFX_SCHEDULER_H__INCLUDED_

#pragma once

//////////////////////////////////////////////////////////////////////
// CScheduleItem class: Represents a scheduled task
//////////////////////////////////////////////////////////////////////

class CScheduleItem
{
// Construction
public:
	CScheduleItem();
	virtual ~CScheduleItem();

// Attributes
public:
	bool		m_bEveryDay;
	int			m_nAction;
	CString		m_sDescription;
	CTime  m_tScheduleDateTime;
	CString		m_sDate;
	CString		m_sTime;
	bool		m_bActive;
	bool		m_bExecuted;

// Operations
public:
	void		Serialize(CArchive& ar, int nVersion);
};

//////////////////////////////////////////////////////////////////////
// CScheduler class: Controls scheduler operations
//////////////////////////////////////////////////////////////////////

class CScheduler  
{
// Construction
public:
	CScheduler();
	virtual ~CScheduler();
	
// Attributes
protected:
	CPtrList	ScheduleList;

// Operations
public:
	POSITION		GetIterator() const;
	CScheduleItem*	GetNext(POSITION& pos) const;
	int				GetCount();
	BOOL			Check(CScheduleItem* pRule) const;
	void			CheckSchedule();
	void			DoScheduledItem(int nAction);
	void			HangUpConnection();
	bool			IsEnabled();
	bool			ShutDownComputer();
	bool			SetShutdownRights();
	bool			ScheduledTimePassed(CTime *pNow,CTime *pThen);

	void			Add(CScheduleItem* pRule);
	void			Remove(CScheduleItem* pRule);

	void			Clear();
	bool			Load();
	bool			Save();
protected:
	void			Serialize(CArchive& ar);
};
enum
{
	SCHEDULE_FULL_SPEED = 1, SCHEDULE_LIMITED_SPEED, SCHEDULE_OFF, SCHEDULE_DISCONNECT, SCHEDULE_EXIT, SCHEDULE_SHUTDOWN  
};

extern CScheduler Scheduler;

#endif // !defined(AFX_SCHEDULER_H__INCLUDED_)
